import { Test, TestingModule } from '@nestjs/testing';
import { generateKeyPair, KeyObject } from 'crypto';
import { getClientToken } from 'nestjs-etcd';
import { promisify } from 'util';
import { JwtService } from './jwt.service';
import { SignJWT } from 'jose/jwt/sign';

describe('JwtService', () => {
  let service: JwtService;
  let privateKey: KeyObject;
  let publicKey: KeyObject;

  beforeAll(async () => {
    const pair = await promisify(generateKeyPair)('ec', {
      namedCurve: 'prime256v1',
    });
    privateKey = pair.privateKey;
    publicKey = pair.publicKey;
  });

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        JwtService,
        {
          provide: getClientToken(),
          useValue: {
            get: () => ({
              buffer: () =>
                Promise.resolve(
                  publicKey.export({ format: 'pem', type: 'spki' }),
                ),
            }),
          },
        },
      ],
    }).compile();

    service = module.get<JwtService>(JwtService);
    await service.onModuleInit();
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('verify', () => {
    it('normal', async () => {
      const token = await new SignJWT({ userId: 'test' })
        .setProtectedHeader({ alg: 'ES256' })
        .setIssuedAt()
        .setIssuer('urn:example:issuer')
        .setAudience('urn:example:audience')
        .setExpirationTime('1h')
        .sign(privateKey);
      await expect(service.verify(token)).resolves.toBeTruthy();
    });
  });
});
