import { OnModuleInit } from '@nestjs/common';
import { Injectable } from '@nestjs/common';
import { KeyObject, createPublicKey } from 'crypto';
import { jwtVerify } from 'jose/jwt/verify';
import { Etcd3, InjectClient } from 'nestjs-etcd';

@Injectable()
export class JwtService implements OnModuleInit {
  publicKey: KeyObject;
  constructor(@InjectClient() private readonly etcd: Etcd3) {}
  async onModuleInit() {
    const buff = await this.etcd
      .get('commons/auth-jwt-public-key/index')
      .buffer();
    this.publicKey = createPublicKey(buff);
  }

  async verify(token: string) {
    return await jwtVerify(token, this.publicKey, {
      algorithms: ['PS256', 'ES256'],
    });
  }
}
